==> hermes-6.0.3.2.rec2.tar.gz: hermes-6.0.3.2.rec2/app/controllers/hermes/api/dmo/entity_reservations_controller.rb <==
module Hermes
  module Api
    module Dmo
      class EntityReservationsController < Hermes::Api::Dmo::ApplicationController

        class << self
          def mutate_params
            {
              name: { type: 'String'},
              from: { type: 'DateTime'},
              to: { type: 'DateTime'},
              housing_offer_id: { type: 'String'},
              entity_id: { type: 'String'},
            }
          end 
      
          def acceptable_params
            {
              is_active: { type: 'Boolean'},
            }.merge(self.mutate_params).merge(super)
          end 
      
          def regex_params
            {
              name: { type: 'String'},
            }
          end
        end
        
        def index
          acceptable_params *self.class.acceptable_params.keys
          regex_params *self.class.regex_params.keys
          
          paginate(
            index_aggregation,
            each_serializer: @serializer,
            **@options
          )
        end

        def create 
          @instance = @model.new(mutate_valid_params.merge(user_id: @user.id))
          find_and_update_same_instance

          return respond_with_errors(@instance) unless @instance.save
          render(jsonapi: @instance, serializer: @serializer, status: :created)
        end 

        def update
          return render_json_error(:not_found, :instance_not_found) unless @instance

          @instance.assign_attributes(mutate_valid_params.merge(user_id: @user.id))
          find_and_update_same_instance

          super
        end 

        def destroy
          @instance = @model.find(params[:id])
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render(json: @instance.errors, status: :unprocessable_entity) unless @instance.destroy
  
          render(body: nil, status: :no_content)
        end

        def update_offer_reservations
          return render_json_error(:not_found, :housing_offer_not_found) unless (offer = Hermes::HousingOffer.find(params[:offer_id]))
          return render_json_error(:unprocessable_entity, :entity_reservations_cannot_be_missing) unless params.has_key?(:entity_reservations)
          params[:entity_reservations] = [] if params[:entity_reservations].nil?

          json = { created_reservations: [], not_created_reservations: [] }
        
          params[:entity_reservations].each do |param_reservation|
            res = generate_reservation_from_offer(offer, param_reservation)
            if res[:response][:return_code] == 200
              json[:created_reservations] << res
            else
              json[:not_created_reservations] << res
            end
          end
        
          entity_ids = params[:entity_reservations].map { |er| er[:entity_id] }
          Hermes::EntityReservation.where(housing_offer: offer, :entity_id.nin => entity_ids).destroy_all
        
          code = json[:not_created_reservations].any? ? 207 : 200

          render(json: json, status: code)
        end
        
        private
        
        def generate_reservation_from_offer(offer, param_reservation)

          unless Hermes::Entity.find(param_reservation[:entity_id])
            return {
              entity_id: param_reservation[:entity_id],
              response: {
                return_code: 404,
                details: 'Entity not found',
              }
            }
          end 

          if (reservation = Hermes::EntityReservation.find_by(housing_offer: offer, entity_id: param_reservation[:entity_id]))
            reservation.update!(from: param_reservation[:from], to: param_reservation[:to])
          else
            reservation = Hermes::EntityReservation.create!(
              housing_offer: offer,
              entity_id: param_reservation[:entity_id],
              from: param_reservation[:from],
              to: param_reservation[:to]
            )
          end
        
          {
            id: reservation.id,
            entity_id: param_reservation[:entity_id],
            response: {
              return_code: 200,
              details: 'Opérations effectuées'
            },
            type: 'entity_reservations',
            attributes: Hermes::EntityReservationSerializer.new(reservation).as_json.except(:id, :type, :relationships, :links),
          }
        rescue StandardError => e
          {
            entity_id: param_reservation[:entity_id],
            response: {
              return_code: 422,
              details: 'Opérations incohérentes',
              message: e.message
            }
          }
        end
        

        def find_and_update_same_instance
          found_instance = Hermes::EntityReservation.order_by(created_at: :desc).find_by(
            entity_id: @instance.entity_id, 
            housing_offer_id: @instance.housing_offer_id,
          )

          if found_instance
            @instance = found_instance
            @instance.assign_attributes(mutate_valid_params.merge(user_id: @user.id))
          end  
        end 

      end
    end
  end
end

==> hermes-6.0.3.5.rec2.tar.gz: hermes-6.0.3.5.rec2/app/controllers/hermes/api/dmo/entity_reservations_controller.rb <==
module Hermes
  module Api
    module Dmo
      class EntityReservationsController < Hermes::Api::Dmo::ApplicationController

        class << self
          def mutate_params
            {
              name: { type: 'String'},
              from: { type: 'DateTime'},
              to: { type: 'DateTime'},
              housing_offer_id: { type: 'String'},
              entity_id: { type: 'String'},
            }
          end 
      
          def acceptable_params
            {
              is_active: { type: 'Boolean'},
            }.merge(self.mutate_params).merge(super)
          end 
      
          def valid_operators
            %w[$in $nin $ne $regex]
          end

          def regex_params
            {
              name: { type: 'String'},
            }
          end
        end
        
        def index
          acceptable_params *self.class.acceptable_params.keys
          regex_params *self.class.regex_params.keys
          valid_operators *self.class.valid_operators
          
          paginate(
            index_aggregation,
            each_serializer: @serializer,
            **@options
          )
        end

        def create 
          @instance = @model.new(mutate_valid_params.merge(user_id: @user.id))
          find_and_update_same_instance

          return respond_with_errors(@instance) unless @instance.save
          render(jsonapi: @instance, serializer: @serializer, status: :created)
        end 

        def update
          return render_json_error(:not_found, :instance_not_found) unless @instance

          @instance.assign_attributes(mutate_valid_params.merge(user_id: @user.id))
          find_and_update_same_instance

          super
        end 

        def destroy
          @instance = @model.find(params[:id])
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render(json: @instance.errors, status: :unprocessable_entity) unless @instance.destroy
  
          render(body: nil, status: :no_content)
        end

        def update_offer_reservations
          return render_json_error(:not_found, :housing_offer_not_found) unless (offer = Hermes::HousingOffer.find(params[:offer_id]))
          return render_json_error(:unprocessable_entity, :entity_reservations_cannot_be_missing) if params[:entity_reservations].nil? || !params[:entity_reservations].is_a?(Array)
        
          json = { created_reservations: [], not_created_reservations: [] }
        
          params[:entity_reservations].each do |param_reservation|
            res = generate_reservation_from_offer(offer, param_reservation)
            if res[:response][:return_code] == 200
              json[:created_reservations] << res
            else
              json[:not_created_reservations] << res
            end
          end
        
          entity_ids = params[:entity_reservations].map { |er| er[:entity_id] }
          Hermes::EntityReservation.where(housing_offer: offer, :entity_id.nin => entity_ids).destroy_all
        
          code = json[:not_created_reservations].any? ? 207 : 200

          render(json: json, status: code)
        end
        
        private
        
        def generate_reservation_from_offer(offer, param_reservation)

          unless Hermes::Entity.find(param_reservation[:entity_id])
            return {
              entity_id: param_reservation[:entity_id],
              response: {
                return_code: 404,
                details: 'Entity not found',
              }
            }
          end 

          if (reservation = Hermes::EntityReservation.find_by(housing_offer: offer, entity_id: param_reservation[:entity_id]))
            reservation.update!(from: param_reservation[:from], to: param_reservation[:to])
          else
            reservation = Hermes::EntityReservation.create!(
              housing_offer: offer,
              entity_id: param_reservation[:entity_id],
              from: param_reservation[:from],
              to: param_reservation[:to]
            )
          end
        
          {
            id: reservation.id,
            entity_id: param_reservation[:entity_id],
            response: {
              return_code: 200,
              details: 'Opérations effectuées'
            },
            type: 'entity_reservations',
            attributes: Hermes::EntityReservationSerializer.new(reservation).as_json.except(:id, :type, :relationships, :links),
          }
        rescue StandardError => e
          {
            entity_id: param_reservation[:entity_id],
            response: {
              return_code: 422,
              details: 'Opérations incohérentes',
              message: e.message
            }
          }
        end
        

        def find_and_update_same_instance
          found_instance = Hermes::EntityReservation.order_by(created_at: :desc).find_by(
            entity_id: @instance.entity_id, 
            housing_offer_id: @instance.housing_offer_id,
          )

          if found_instance
            @instance = found_instance
            @instance.assign_attributes(mutate_valid_params.merge(user_id: @user.id))
          end  
        end 

      end
    end
  end
end
